# This file was auto-generated by Fern from our API Definition.

ConnectionJwksUri = str
"""
URL of the identity provider's JSON Web Key Set (JWKS) endpoint containing public keys for signature verification. Auth0 retrieves these keys to validate ID token signatures.
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.create_verification_email_response_content import CreateVerificationEmailResponseContent
from ...types.identity import Identity
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVerificationEmailClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateVerificationEmailResponseContent]:
        """
        Send an email to the specified user that asks them to click a link to [verify their email address](https://auth0.com/docs/email/custom#verification-email).

        Note: You must have the `Status` toggle enabled for the verification email template for the email to be sent.

        Parameters
        ----------
        user_id : str
            user_id of the user to send the verification email to.

        client_id : typing.Optional[str]
            client_id of the client (application). If no value provided, the global Client ID will be used.

        identity : typing.Optional[Identity]

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateVerificationEmailResponseContent]
            Job successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "jobs/verification-email",
            method="POST",
            json={
                "user_id": user_id,
                "client_id": client_id,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=Identity, direction="write"
                ),
                "organization_id": organization_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVerificationEmailResponseContent,
                    parse_obj_as(
                        type_=CreateVerificationEmailResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVerificationEmailClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateVerificationEmailResponseContent]:
        """
        Send an email to the specified user that asks them to click a link to [verify their email address](https://auth0.com/docs/email/custom#verification-email).

        Note: You must have the `Status` toggle enabled for the verification email template for the email to be sent.

        Parameters
        ----------
        user_id : str
            user_id of the user to send the verification email to.

        client_id : typing.Optional[str]
            client_id of the client (application). If no value provided, the global Client ID will be used.

        identity : typing.Optional[Identity]

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateVerificationEmailResponseContent]
            Job successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "jobs/verification-email",
            method="POST",
            json={
                "user_id": user_id,
                "client_id": client_id,
                "identity": convert_and_respect_annotation_metadata(
                    object_=identity, annotation=Identity, direction="write"
                ),
                "organization_id": organization_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateVerificationEmailResponseContent,
                    parse_obj_as(
                        type_=CreateVerificationEmailResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_member_deleted_object_group_0 import (
    EventStreamCloudEventGroupMemberDeletedObjectGroup0,
)
from .event_stream_cloud_event_group_member_deleted_object_group_1 import (
    EventStreamCloudEventGroupMemberDeletedObjectGroup1,
)
from .event_stream_cloud_event_group_member_deleted_object_group_2 import (
    EventStreamCloudEventGroupMemberDeletedObjectGroup2,
)

EventStreamCloudEventGroupMemberDeletedObjectGroup = typing.Union[
    EventStreamCloudEventGroupMemberDeletedObjectGroup0,
    EventStreamCloudEventGroupMemberDeletedObjectGroup1,
    EventStreamCloudEventGroupMemberDeletedObjectGroup2,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_upstream_additional_properties import ConnectionUpstreamAdditionalProperties

ConnectionUpstreamParamsFacebook = typing.Dict[str, ConnectionUpstreamAdditionalProperties]
"""
Options for adding parameters in the request to the upstream IdP. See https://auth0.com/docs/authenticate/identity-providers/pass-parameters-to-idps
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_context import EventStreamCloudEventContext
from .event_stream_cloud_event_group_created_object import EventStreamCloudEventGroupCreatedObject


class EventStreamCloudEventGroupCreatedData(UniversalBaseModel):
    """
    The event payload.
    """

    object: EventStreamCloudEventGroupCreatedObject
    context: typing.Optional[EventStreamCloudEventContext] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.action import Action
from ..types.action_module_reference import ActionModuleReference
from ..types.action_secret_request import ActionSecretRequest
from ..types.action_trigger import ActionTrigger
from ..types.action_trigger_type_enum import ActionTriggerTypeEnum
from ..types.action_version_dependency import ActionVersionDependency
from ..types.create_action_response_content import CreateActionResponseContent
from ..types.deploy_action_response_content import DeployActionResponseContent
from ..types.get_action_response_content import GetActionResponseContent
from ..types.list_actions_paginated_response_content import ListActionsPaginatedResponseContent
from ..types.test_action_payload import TestActionPayload
from ..types.test_action_response_content import TestActionResponseContent
from ..types.update_action_response_content import UpdateActionResponseContent
from .raw_client import AsyncRawActionsClient, RawActionsClient

if typing.TYPE_CHECKING:
    from .executions.client import AsyncExecutionsClient, ExecutionsClient
    from .modules.client import AsyncModulesClient, ModulesClient
    from .triggers.client import AsyncTriggersClient, TriggersClient
    from .versions.client import AsyncVersionsClient, VersionsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawActionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._versions: typing.Optional[VersionsClient] = None
        self._executions: typing.Optional[ExecutionsClient] = None
        self._modules: typing.Optional[ModulesClient] = None
        self._triggers: typing.Optional[TriggersClient] = None

    @property
    def with_raw_response(self) -> RawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawActionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        trigger_id: typing.Optional[ActionTriggerTypeEnum] = None,
        action_name: typing.Optional[str] = None,
        deployed: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        installed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Action, ListActionsPaginatedResponseContent]:
        """
        Retrieve all actions.

        Parameters
        ----------
        trigger_id : typing.Optional[ActionTriggerTypeEnum]
            An actions extensibility point.

        action_name : typing.Optional[str]
            The name of the action to retrieve.

        deployed : typing.Optional[bool]
            Optional filter to only retrieve actions that are deployed.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in single response. 20 by default

        installed : typing.Optional[bool]
            Optional. When true, return only installed actions. When false, return only custom actions. Returns all actions by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Action, ListActionsPaginatedResponseContent]
            The actions were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.actions.list(
            trigger_id="post-login",
            action_name="actionName",
            deployed=True,
            page=1,
            per_page=1,
            installed=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            trigger_id=trigger_id,
            action_name=action_name,
            deployed=deployed,
            page=page,
            per_page=per_page,
            installed=installed,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        supported_triggers: typing.Sequence[ActionTrigger],
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        deploy: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateActionResponseContent:
        """
        Create an action. Once an action is created, it must be deployed, and then bound to a trigger before it will be executed as part of a flow.

        Parameters
        ----------
        name : str
            The name of an action.

        supported_triggers : typing.Sequence[ActionTrigger]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        deploy : typing.Optional[bool]
            True if the action should be deployed after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionResponseContent
            Action successfully created.

        Examples
        --------
        from auth0 import ActionTrigger, Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.create(
            name="name",
            supported_triggers=[
                ActionTrigger(
                    id="post-login",
                )
            ],
        )
        """
        _response = self._raw_client.create(
            name=name,
            supported_triggers=supported_triggers,
            code=code,
            dependencies=dependencies,
            runtime=runtime,
            secrets=secrets,
            modules=modules,
            deploy=deploy,
            request_options=request_options,
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetActionResponseContent:
        """
        Retrieve an action by its ID.

        Parameters
        ----------
        id : str
            The ID of the action to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionResponseContent
            The action was retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(
        self, id: str, *, force: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes an action and all of its associated versions. An action must be unbound from all triggers before it can be deleted.

        Parameters
        ----------
        id : str
            The ID of the action to delete.

        force : typing.Optional[bool]
            Force action deletion detaching bindings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.delete(
            id="id",
            force=True,
        )
        """
        _response = self._raw_client.delete(id, force=force, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        supported_triggers: typing.Optional[typing.Sequence[ActionTrigger]] = OMIT,
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateActionResponseContent:
        """
        Update an existing action. If this action is currently bound to a trigger, updating it will **not** affect any user flows until the action is deployed.

        Parameters
        ----------
        id : str
            The id of the action to update.

        name : typing.Optional[str]
            The name of an action.

        supported_triggers : typing.Optional[typing.Sequence[ActionTrigger]]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateActionResponseContent
            Action successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            supported_triggers=supported_triggers,
            code=code,
            dependencies=dependencies,
            runtime=runtime,
            secrets=secrets,
            modules=modules,
            request_options=request_options,
        )
        return _response.data

    def deploy(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeployActionResponseContent:
        """
        Deploy an action. Deploying an action will create a new immutable version of the action. If the action is currently bound to a trigger, then the system will begin executing the newly deployed version of the action immediately. Otherwise, the action will only be executed as a part of a flow once it is bound to that flow.

        Parameters
        ----------
        id : str
            The ID of an action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeployActionResponseContent
            Request to create action version was accepted.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.deploy(
            id="id",
        )
        """
        _response = self._raw_client.deploy(id, request_options=request_options)
        return _response.data

    def test(
        self, id: str, *, payload: TestActionPayload, request_options: typing.Optional[RequestOptions] = None
    ) -> TestActionResponseContent:
        """
        Test an action. After updating an action, it can be tested prior to being deployed to ensure it behaves as expected.

        Parameters
        ----------
        id : str
            The id of the action to test.

        payload : TestActionPayload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestActionResponseContent
            Test action version successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.test(
            id="id",
            payload={"key": "value"},
        )
        """
        _response = self._raw_client.test(id, payload=payload, request_options=request_options)
        return _response.data

    @property
    def versions(self):
        if self._versions is None:
            from .versions.client import VersionsClient  # noqa: E402

            self._versions = VersionsClient(client_wrapper=self._client_wrapper)
        return self._versions

    @property
    def executions(self):
        if self._executions is None:
            from .executions.client import ExecutionsClient  # noqa: E402

            self._executions = ExecutionsClient(client_wrapper=self._client_wrapper)
        return self._executions

    @property
    def modules(self):
        if self._modules is None:
            from .modules.client import ModulesClient  # noqa: E402

            self._modules = ModulesClient(client_wrapper=self._client_wrapper)
        return self._modules

    @property
    def triggers(self):
        if self._triggers is None:
            from .triggers.client import TriggersClient  # noqa: E402

            self._triggers = TriggersClient(client_wrapper=self._client_wrapper)
        return self._triggers


class AsyncActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawActionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._versions: typing.Optional[AsyncVersionsClient] = None
        self._executions: typing.Optional[AsyncExecutionsClient] = None
        self._modules: typing.Optional[AsyncModulesClient] = None
        self._triggers: typing.Optional[AsyncTriggersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawActionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawActionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        trigger_id: typing.Optional[ActionTriggerTypeEnum] = None,
        action_name: typing.Optional[str] = None,
        deployed: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        installed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Action, ListActionsPaginatedResponseContent]:
        """
        Retrieve all actions.

        Parameters
        ----------
        trigger_id : typing.Optional[ActionTriggerTypeEnum]
            An actions extensibility point.

        action_name : typing.Optional[str]
            The name of the action to retrieve.

        deployed : typing.Optional[bool]
            Optional filter to only retrieve actions that are deployed.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in single response. 20 by default

        installed : typing.Optional[bool]
            Optional. When true, return only installed actions. When false, return only custom actions. Returns all actions by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Action, ListActionsPaginatedResponseContent]
            The actions were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.actions.list(
                trigger_id="post-login",
                action_name="actionName",
                deployed=True,
                page=1,
                per_page=1,
                installed=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            trigger_id=trigger_id,
            action_name=action_name,
            deployed=deployed,
            page=page,
            per_page=per_page,
            installed=installed,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        supported_triggers: typing.Sequence[ActionTrigger],
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        deploy: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateActionResponseContent:
        """
        Create an action. Once an action is created, it must be deployed, and then bound to a trigger before it will be executed as part of a flow.

        Parameters
        ----------
        name : str
            The name of an action.

        supported_triggers : typing.Sequence[ActionTrigger]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        deploy : typing.Optional[bool]
            True if the action should be deployed after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionResponseContent
            Action successfully created.

        Examples
        --------
        import asyncio

        from auth0 import ActionTrigger, AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.create(
                name="name",
                supported_triggers=[
                    ActionTrigger(
                        id="post-login",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            supported_triggers=supported_triggers,
            code=code,
            dependencies=dependencies,
            runtime=runtime,
            secrets=secrets,
            modules=modules,
            deploy=deploy,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionResponseContent:
        """
        Retrieve an action by its ID.

        Parameters
        ----------
        id : str
            The ID of the action to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionResponseContent
            The action was retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, *, force: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes an action and all of its associated versions. An action must be unbound from all triggers before it can be deleted.

        Parameters
        ----------
        id : str
            The ID of the action to delete.

        force : typing.Optional[bool]
            Force action deletion detaching bindings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.delete(
                id="id",
                force=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, force=force, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        supported_triggers: typing.Optional[typing.Sequence[ActionTrigger]] = OMIT,
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateActionResponseContent:
        """
        Update an existing action. If this action is currently bound to a trigger, updating it will **not** affect any user flows until the action is deployed.

        Parameters
        ----------
        id : str
            The id of the action to update.

        name : typing.Optional[str]
            The name of an action.

        supported_triggers : typing.Optional[typing.Sequence[ActionTrigger]]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateActionResponseContent
            Action successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            supported_triggers=supported_triggers,
            code=code,
            dependencies=dependencies,
            runtime=runtime,
            secrets=secrets,
            modules=modules,
            request_options=request_options,
        )
        return _response.data

    async def deploy(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeployActionResponseContent:
        """
        Deploy an action. Deploying an action will create a new immutable version of the action. If the action is currently bound to a trigger, then the system will begin executing the newly deployed version of the action immediately. Otherwise, the action will only be executed as a part of a flow once it is bound to that flow.

        Parameters
        ----------
        id : str
            The ID of an action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeployActionResponseContent
            Request to create action version was accepted.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.deploy(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deploy(id, request_options=request_options)
        return _response.data

    async def test(
        self, id: str, *, payload: TestActionPayload, request_options: typing.Optional[RequestOptions] = None
    ) -> TestActionResponseContent:
        """
        Test an action. After updating an action, it can be tested prior to being deployed to ensure it behaves as expected.

        Parameters
        ----------
        id : str
            The id of the action to test.

        payload : TestActionPayload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TestActionResponseContent
            Test action version successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.test(
                id="id",
                payload={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.test(id, payload=payload, request_options=request_options)
        return _response.data

    @property
    def versions(self):
        if self._versions is None:
            from .versions.client import AsyncVersionsClient  # noqa: E402

            self._versions = AsyncVersionsClient(client_wrapper=self._client_wrapper)
        return self._versions

    @property
    def executions(self):
        if self._executions is None:
            from .executions.client import AsyncExecutionsClient  # noqa: E402

            self._executions = AsyncExecutionsClient(client_wrapper=self._client_wrapper)
        return self._executions

    @property
    def modules(self):
        if self._modules is None:
            from .modules.client import AsyncModulesClient  # noqa: E402

            self._modules = AsyncModulesClient(client_wrapper=self._client_wrapper)
        return self._modules

    @property
    def triggers(self):
        if self._triggers is None:
            from .triggers.client import AsyncTriggersClient  # noqa: E402

            self._triggers = AsyncTriggersClient(client_wrapper=self._client_wrapper)
        return self._triggers



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_trigger import ActionTrigger


class ActionBase(UniversalBaseModel):
    """
    The action to which this version belongs.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID of the action.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of an action.
    """

    supported_triggers: typing.Optional[typing.List[ActionTrigger]] = pydantic.Field(default=None)
    """
    The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.
    """

    all_changes_deployed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if all of an Action's contents have been deployed.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this action was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this action was updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CimdMappedPrivateKeyJwtCredential(UniversalBaseModel):
    credential_type: str = pydantic.Field()
    """
    Type of credential (e.g., public_key)
    """

    kid: str = pydantic.Field()
    """
    Key identifier from JWKS or calculated thumbprint
    """

    alg: str = pydantic.Field()
    """
    Algorithm (e.g., RS256, RS384, PS256)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FlowsVaultConnectionSetupTypeOauthClientCredentialsEnum = typing.Union[
    typing.Literal["OAUTH_CLIENT_CREDENTIALS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from typing import Any, Dict, Optional


class ParsingError(Exception):
    """
    Raised when the SDK fails to parse/validate a response from the server.
    This typically indicates that the server returned a response whose shape
    does not match the expected schema.
    """

    headers: Optional[Dict[str, str]]
    status_code: Optional[int]
    body: Any
    cause: Optional[Exception]

    def __init__(
        self,
        *,
        headers: Optional[Dict[str, str]] = None,
        status_code: Optional[int] = None,
        body: Any = None,
        cause: Optional[Exception] = None,
    ) -> None:
        self.headers = headers
        self.status_code = status_code
        self.body = body
        self.cause = cause
        super().__init__()
        if cause is not None:
            self.__cause__ = cause

    def __str__(self) -> str:
        cause_str = f", cause: {self.cause}" if self.cause is not None else ""
        return f"headers: {self.headers}, status_code: {self.status_code}, body: {self.body}{cause_str}"



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_user_updated_object_identities_item_custom import (
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemCustom,
)
from .event_stream_cloud_event_user_updated_object_identities_item_database import (
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemDatabase,
)
from .event_stream_cloud_event_user_updated_object_identities_item_enterprise import (
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemEnterprise,
)
from .event_stream_cloud_event_user_updated_object_identities_item_passwordless import (
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemPasswordless,
)
from .event_stream_cloud_event_user_updated_object_identities_item_social import (
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemSocial,
)

EventStreamCloudEventUserUpdatedObjectIdentitiesItem = typing.Union[
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemCustom,
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemDatabase,
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemEnterprise,
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemPasswordless,
    EventStreamCloudEventUserUpdatedObjectIdentitiesItemSocial,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.get_user_groups_paginated_response_content import GetUserGroupsPaginatedResponseContent
from ...types.user_groups_response_schema import UserGroupsResponseSchema
from .raw_client import AsyncRawGroupsClient, RawGroupsClient


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserGroupsResponseSchema, GetUserGroupsPaginatedResponseContent]:
        """
        List all groups to which this user belongs.

        Parameters
        ----------
        id : str
            ID of the user to list groups for.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserGroupsResponseSchema, GetUserGroupsPaginatedResponseContent]
            Groups successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.groups.get(
            id="id",
            fields="fields",
            include_fields=True,
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.get(
            id, fields=fields, include_fields=include_fields, from_=from_, take=take, request_options=request_options
        )


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserGroupsResponseSchema, GetUserGroupsPaginatedResponseContent]:
        """
        List all groups to which this user belongs.

        Parameters
        ----------
        id : str
            ID of the user to list groups for.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserGroupsResponseSchema, GetUserGroupsPaginatedResponseContent]
            Groups successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.groups.get(
                id="id",
                fields="fields",
                include_fields=True,
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.get(
            id, fields=fields, include_fields=include_fields, from_=from_, take=take, request_options=request_options
        )



from .conftest import get_client, verify_request_count


def test_clients_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "clients.list_.0"
    client = get_client(test_id)
    client.clients.list(
        fields="fields",
        include_fields=True,
        page=1,
        per_page=1,
        include_totals=True,
        is_global=True,
        is_first_party=True,
        app_type="app_type",
        external_client_id="external_client_id",
        q="q",
    )
    verify_request_count(
        test_id,
        "GET",
        "/clients",
        {
            "fields": "fields",
            "include_fields": "true",
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "is_global": "true",
            "is_first_party": "true",
            "app_type": "app_type",
            "external_client_id": "external_client_id",
            "q": "q",
        },
        1,
    )


def test_clients_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "clients.create.0"
    client = get_client(test_id)
    client.clients.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/clients", None, 1)


def test_clients_preview_cimd_metadata() -> None:
    """Test previewCimdMetadata endpoint with WireMock"""
    test_id = "clients.preview_cimd_metadata.0"
    client = get_client(test_id)
    client.clients.preview_cimd_metadata(
        external_client_id="external_client_id",
    )
    verify_request_count(test_id, "POST", "/clients/cimd/preview", None, 1)


def test_clients_register_cimd_client() -> None:
    """Test registerCimdClient endpoint with WireMock"""
    test_id = "clients.register_cimd_client.0"
    client = get_client(test_id)
    client.clients.register_cimd_client(
        external_client_id="external_client_id",
    )
    verify_request_count(test_id, "POST", "/clients/cimd/register", None, 1)


def test_clients_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "clients.get.0"
    client = get_client(test_id)
    client.clients.get(
        id="id",
        fields="fields",
        include_fields=True,
    )
    verify_request_count(test_id, "GET", "/clients/id", {"fields": "fields", "include_fields": "true"}, 1)


def test_clients_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "clients.delete.0"
    client = get_client(test_id)
    client.clients.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/clients/id", None, 1)


def test_clients_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "clients.update.0"
    client = get_client(test_id)
    client.clients.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/clients/id", None, 1)


def test_clients_rotate_secret() -> None:
    """Test rotateSecret endpoint with WireMock"""
    test_id = "clients.rotate_secret.0"
    client = get_client(test_id)
    client.clients.rotate_secret(
        id="id",
    )
    verify_request_count(test_id, "POST", "/clients/id/rotate-secret", None, 1)



import unittest
from unittest import mock

from auth0.authentication.revoke_token import RevokeToken


class TestRevokeToken(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_revoke_refresh_token(self, mock_post):
        a = RevokeToken("my.domain.com", "cid")

        # regular apps
        a.revoke_refresh_token(token="tkn")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/revoke")
        self.assertEqual(kwargs["data"], {"client_id": "cid", "token": "tkn"})

        # confidential apps
        a = RevokeToken("my.domain.com", "cid", client_secret="sh!")
        a.revoke_refresh_token(token="tkn")

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/revoke")
        self.assertEqual(
            kwargs["data"], {"client_id": "cid", "token": "tkn", "client_secret": "sh!"}
        )



from .conftest import get_client, verify_request_count


def test_refreshTokens_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "refresh_tokens.list_.0"
    client = get_client(test_id)
    client.refresh_tokens.list(
        user_id="user_id",
        client_id="client_id",
        from_="from",
        take=1,
        fields="fields",
        include_fields=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/refresh-tokens",
        {
            "user_id": "user_id",
            "client_id": "client_id",
            "from": "from",
            "take": "1",
            "fields": "fields",
            "include_fields": "true",
        },
        1,
    )


def test_refreshTokens_revoke() -> None:
    """Test revoke endpoint with WireMock"""
    test_id = "refresh_tokens.revoke.0"
    client = get_client(test_id)
    client.refresh_tokens.revoke()
    verify_request_count(test_id, "POST", "/refresh-tokens/revoke", None, 1)


def test_refreshTokens_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "refresh_tokens.get.0"
    client = get_client(test_id)
    client.refresh_tokens.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/refresh-tokens/id", None, 1)


def test_refreshTokens_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "refresh_tokens.delete.0"
    client = get_client(test_id)
    client.refresh_tokens.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/refresh-tokens/id", None, 1)


def test_refreshTokens_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "refresh_tokens.update.0"
    client = get_client(test_id)
    client.refresh_tokens.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/refresh-tokens/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

ConnectionClientIdGoogleOAuth2 = str
"""
Your Google OAuth 2.0 client ID. You can find this in your [Google Cloud Console](https://console.cloud.google.com/apis/credentials) under the OAuth 2.0 Client IDs section.
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionAdminAccessTokenGoogleApps = str
"""
Google Workspace admin access token used to retrieve extended user attributes (such as group memberships, admin status, and suspension state) from the [Google Directory API](https://developers.google.com/admin-sdk/directory). This token is automatically managed by Auth0.
"""



from .conftest import get_client, verify_request_count


def test_guardian_factors_sms_get_twilio_provider() -> None:
    """Test getTwilioProvider endpoint with WireMock"""
    test_id = "guardian.factors.sms.get_twilio_provider.0"
    client = get_client(test_id)
    client.guardian.factors.sms.get_twilio_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/sms/providers/twilio", None, 1)


def test_guardian_factors_sms_set_twilio_provider() -> None:
    """Test setTwilioProvider endpoint with WireMock"""
    test_id = "guardian.factors.sms.set_twilio_provider.0"
    client = get_client(test_id)
    client.guardian.factors.sms.set_twilio_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/sms/providers/twilio", None, 1)


def test_guardian_factors_sms_get_selected_provider() -> None:
    """Test getSelectedProvider endpoint with WireMock"""
    test_id = "guardian.factors.sms.get_selected_provider.0"
    client = get_client(test_id)
    client.guardian.factors.sms.get_selected_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/sms/selected-provider", None, 1)


def test_guardian_factors_sms_set_provider() -> None:
    """Test setProvider endpoint with WireMock"""
    test_id = "guardian.factors.sms.set_provider.0"
    client = get_client(test_id)
    client.guardian.factors.sms.set_provider(
        provider="auth0",
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/sms/selected-provider", None, 1)


def test_guardian_factors_sms_get_templates() -> None:
    """Test getTemplates endpoint with WireMock"""
    test_id = "guardian.factors.sms.get_templates.0"
    client = get_client(test_id)
    client.guardian.factors.sms.get_templates()
    verify_request_count(test_id, "GET", "/guardian/factors/sms/templates", None, 1)


def test_guardian_factors_sms_set_templates() -> None:
    """Test setTemplates endpoint with WireMock"""
    test_id = "guardian.factors.sms.set_templates.0"
    client = get_client(test_id)
    client.guardian.factors.sms.set_templates(
        enrollment_message="enrollment_message",
        verification_message="verification_message",
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/sms/templates", None, 1)



# This file was auto-generated by Fern from our API Definition.

ConnectionUserIdAttributeSaml = str
"""
Custom SAML assertion attribute to use as the unique user identifier. When provided, this attribute is prepended to the default user_id mapping list with highest priority. Accepts a string (single SAML attribute name).
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionDisableSelfServiceChangePassword = bool
"""
Indicates whether to disable self-service change password. Set to true to stop the "Forgot Password" being displayed on login pages
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientAddonDropbox = typing.Dict[str, typing.Any]
"""
Dropbox SSO indicator (no configuration settings needed for Dropbox SSO).
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConnectionCustomScripts(UniversalBaseModel):
    """
    A map of scripts used to integrate with a custom database.
    """

    login: typing.Optional[str] = None
    get_user: typing.Optional[str] = None
    delete: typing.Optional[str] = None
    change_password: typing.Optional[str] = None
    verify: typing.Optional[str] = None
    create: typing.Optional[str] = None
    change_username: typing.Optional[str] = None
    change_email: typing.Optional[str] = None
    change_phone_number: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .session_date import SessionDate


class SessionAuthenticationSignal(UniversalBaseModel):
    """
    Authentication signal details
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    One of: "federated", "passkey", "pwd", "sms", "email", "mfa", "mock" or a custom method denoted by a URL
    """

    timestamp: typing.Optional[SessionDate] = None
    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    A specific MFA factor. Only present when "name" is set to "mfa"
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .breached_password_detection_admin_notification_frequency_enum import (
    BreachedPasswordDetectionAdminNotificationFrequencyEnum,
)
from .breached_password_detection_method_enum import BreachedPasswordDetectionMethodEnum
from .breached_password_detection_shields_enum import BreachedPasswordDetectionShieldsEnum
from .breached_password_detection_stage import BreachedPasswordDetectionStage


class UpdateBreachedPasswordDetectionSettingsResponseContent(UniversalBaseModel):
    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether or not breached password detection is active.
    """

    shields: typing.Optional[typing.List[BreachedPasswordDetectionShieldsEnum]] = pydantic.Field(default=None)
    """
    Action to take when a breached password is detected during a login.
          Possible values: <code>block</code>, <code>user_notification</code>, <code>admin_notification</code>.
    """

    admin_notification_frequency: typing.Optional[
        typing.List[BreachedPasswordDetectionAdminNotificationFrequencyEnum]
    ] = pydantic.Field(default=None)
    """
    When "admin_notification" is enabled, determines how often email notifications are sent.
            Possible values: <code>immediately</code>, <code>daily</code>, <code>weekly</code>, <code>monthly</code>.
    """

    method: typing.Optional[BreachedPasswordDetectionMethodEnum] = None
    stage: typing.Optional[BreachedPasswordDetectionStage] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .organization_discovery_domain_status import OrganizationDiscoveryDomainStatus


class CreateOrganizationDiscoveryDomainResponseContent(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Organization discovery domain identifier.
    """

    domain: str = pydantic.Field()
    """
    The domain name to associate with the organization e.g. acme.com.
    """

    status: OrganizationDiscoveryDomainStatus
    use_for_organization_discovery: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether this domain should be used for organization discovery.
    """

    verification_txt: str = pydantic.Field()
    """
    A unique token generated for the discovery domain. This must be placed in a DNS TXT record at the location specified by the verification_host field to prove domain ownership.
    """

    verification_host: str = pydantic.Field()
    """
    The full domain where the TXT record should be added.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_group_deleted_object_0_type_enum import EventStreamCloudEventGroupDeletedObject0TypeEnum


class EventStreamCloudEventGroupDeletedObject0(UniversalBaseModel):
    """
    Connection group with updated_at timestamp
    """

    id: str = pydantic.Field()
    """
    The unique identifier for the group.
    """

    name: str = pydantic.Field()
    """
    The name of the group.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The external identifier for the group.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Date and time when this entity was created (ISO_8601 format).
    """

    type: EventStreamCloudEventGroupDeletedObject0TypeEnum
    connection_id: str = pydantic.Field()
    """
    The connection ID associated with the group.
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Date and time when this entity was last updated/modified (ISO_8601 format).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormFieldFileConfigCategoryEnum = typing.Union[
    typing.Literal["AUDIO", "VIDEO", "IMAGE", "DOCUMENT", "ARCHIVE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .attack_protection_captcha_arkose_response_content import AttackProtectionCaptchaArkoseResponseContent
from .attack_protection_captcha_auth_challenge_response_content import (
    AttackProtectionCaptchaAuthChallengeResponseContent,
)
from .attack_protection_captcha_friendly_captcha_response_content import (
    AttackProtectionCaptchaFriendlyCaptchaResponseContent,
)
from .attack_protection_captcha_hcaptcha_response_content import AttackProtectionCaptchaHcaptchaResponseContent
from .attack_protection_captcha_recaptcha_enterprise_response_content import (
    AttackProtectionCaptchaRecaptchaEnterpriseResponseContent,
)
from .attack_protection_captcha_recaptcha_v_2_response_content import AttackProtectionCaptchaRecaptchaV2ResponseContent
from .attack_protection_captcha_simple_captcha_response_content import (
    AttackProtectionCaptchaSimpleCaptchaResponseContent,
)


class UpdateAttackProtectionCaptchaResponseContent(UniversalBaseModel):
    active_provider_id: typing.Optional[str] = None
    arkose: typing.Optional[AttackProtectionCaptchaArkoseResponseContent] = None
    auth_challenge: typing.Optional[AttackProtectionCaptchaAuthChallengeResponseContent] = None
    hcaptcha: typing.Optional[AttackProtectionCaptchaHcaptchaResponseContent] = None
    friendly_captcha: typing.Optional[AttackProtectionCaptchaFriendlyCaptchaResponseContent] = None
    recaptcha_enterprise: typing.Optional[AttackProtectionCaptchaRecaptchaEnterpriseResponseContent] = None
    recaptcha_v_2: typing_extensions.Annotated[
        typing.Optional[AttackProtectionCaptchaRecaptchaV2ResponseContent],
        FieldMetadata(alias="recaptcha_v2"),
        pydantic.Field(alias="recaptcha_v2"),
    ] = None
    simple_captcha: typing.Optional[AttackProtectionCaptchaSimpleCaptchaResponseContent] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.action_version import ActionVersion
from ...types.deploy_action_version_request_content import DeployActionVersionRequestContent
from ...types.deploy_action_version_response_content import DeployActionVersionResponseContent
from ...types.get_action_version_response_content import GetActionVersionResponseContent
from ...types.list_action_versions_paginated_response_content import ListActionVersionsPaginatedResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        action_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]:
        """
        Retrieve all of an action's versions. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            This field specify the maximum number of results to be returned by the server. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]
            The action versions were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionVersionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionVersionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.versions
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    action_id,
                    page=page + 1,
                    per_page=per_page,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, action_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetActionVersionResponseContent]:
        """
        Retrieve a specific version of an action. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        id : str
            The ID of the action version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetActionVersionResponseContent]
            The action version was retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionVersionResponseContent,
                    parse_obj_as(
                        type_=GetActionVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deploy(
        self,
        action_id: str,
        id: str,
        *,
        request: typing.Optional[DeployActionVersionRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DeployActionVersionResponseContent]:
        """
        Performs the equivalent of a roll-back of an action to an earlier, specified version. Creates a new, deployed action version that is identical to the specified version. If this action is currently bound to a trigger, the system will begin executing the newly-created version immediately.

        Parameters
        ----------
        action_id : str
            The ID of an action.

        id : str
            The ID of an action version.

        request : typing.Optional[DeployActionVersionRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeployActionVersionResponseContent]
            Request to create action version was accepted.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions/{encode_path_param(id)}/deploy",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=typing.Optional[DeployActionVersionRequestContent], direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeployActionVersionResponseContent,
                    parse_obj_as(
                        type_=DeployActionVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        action_id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]:
        """
        Retrieve all of an action's versions. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            This field specify the maximum number of results to be returned by the server. 20 by default

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionVersion, ListActionVersionsPaginatedResponseContent]
            The action versions were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionVersionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionVersionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.versions
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        action_id,
                        page=page + 1,
                        per_page=per_page,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, action_id: str, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetActionVersionResponseContent]:
        """
        Retrieve a specific version of an action. An action version is created whenever an action is deployed. An action version is immutable, once created.

        Parameters
        ----------
        action_id : str
            The ID of the action.

        id : str
            The ID of the action version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetActionVersionResponseContent]
            The action version was retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionVersionResponseContent,
                    parse_obj_as(
                        type_=GetActionVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deploy(
        self,
        action_id: str,
        id: str,
        *,
        request: typing.Optional[DeployActionVersionRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DeployActionVersionResponseContent]:
        """
        Performs the equivalent of a roll-back of an action to an earlier, specified version. Creates a new, deployed action version that is identical to the specified version. If this action is currently bound to a trigger, the system will begin executing the newly-created version immediately.

        Parameters
        ----------
        action_id : str
            The ID of an action.

        id : str
            The ID of an action version.

        request : typing.Optional[DeployActionVersionRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeployActionVersionResponseContent]
            Request to create action version was accepted.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(action_id)}/versions/{encode_path_param(id)}/deploy",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=typing.Optional[DeployActionVersionRequestContent], direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeployActionVersionResponseContent,
                    parse_obj_as(
                        type_=DeployActionVersionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

PartialGroupsEnum = typing.Union[
    typing.Literal[
        "login",
        "login-id",
        "login-password",
        "login-passwordless",
        "signup",
        "signup-id",
        "signup-password",
        "customized-consent",
        "passkeys",
        "confirmation",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.precondition_failed_error import PreconditionFailedError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.action_module_action import ActionModuleAction
from ...types.action_module_dependency_request import ActionModuleDependencyRequest
from ...types.action_module_list_item import ActionModuleListItem
from ...types.action_module_secret_request import ActionModuleSecretRequest
from ...types.create_action_module_response_content import CreateActionModuleResponseContent
from ...types.get_action_module_actions_response_content import GetActionModuleActionsResponseContent
from ...types.get_action_module_response_content import GetActionModuleResponseContent
from ...types.get_action_modules_response_content import GetActionModulesResponseContent
from ...types.rollback_action_module_response_content import RollbackActionModuleResponseContent
from ...types.update_action_module_response_content import UpdateActionModuleResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawModulesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleListItem, GetActionModulesResponseContent]:
        """
        Retrieve a paginated list of all Actions Modules with optional filtering and totals.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleListItem, GetActionModulesResponseContent]
            The action modules were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "actions/modules",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModulesResponseContent,
                    parse_obj_as(
                        type_=GetActionModulesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.modules
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    page=page + 1,
                    per_page=per_page,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        code: str,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        publish: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateActionModuleResponseContent]:
        """
        Create a new Actions Module for reusable code across actions.

        Parameters
        ----------
        name : str
            The name of the action module.

        code : str
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        api_version : typing.Optional[str]
            The API version of the module.

        publish : typing.Optional[bool]
            Whether to publish the module immediately after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateActionModuleResponseContent]
            The action module was created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "actions/modules",
            method="POST",
            json={
                "name": name,
                "code": code,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionModuleSecretRequest], direction="write"
                ),
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionModuleDependencyRequest], direction="write"
                ),
                "api_version": api_version,
                "publish": publish,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionModuleResponseContent,
                    parse_obj_as(
                        type_=CreateActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetActionModuleResponseContent]:
        """
        Retrieve details of a specific Actions Module by its unique identifier.

        Parameters
        ----------
        id : str
            The ID of the action module to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetActionModuleResponseContent]
            The action module was retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionModuleResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Permanently delete an Actions Module. This will fail if the module is still in use by any actions.

        Parameters
        ----------
        id : str
            The ID of the Actions Module to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 412:
                raise PreconditionFailedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        code: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateActionModuleResponseContent]:
        """
        Update properties of an existing Actions Module, such as code, dependencies, or secrets.

        Parameters
        ----------
        id : str
            The ID of the action module to update.

        code : typing.Optional[str]
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateActionModuleResponseContent]
            The action module was updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="PATCH",
            json={
                "code": code,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionModuleSecretRequest], direction="write"
                ),
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionModuleDependencyRequest], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionModuleResponseContent,
                    parse_obj_as(
                        type_=UpdateActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_actions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]:
        """
        Lists all actions that are using a specific Actions Module, showing which deployed action versions reference this Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]
            The actions were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/actions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModuleActionsResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleActionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.actions
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list_actions(
                    id,
                    page=page + 1,
                    per_page=per_page,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def rollback(
        self, id: str, *, module_version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RollbackActionModuleResponseContent]:
        """
        Rolls back an Actions Module's draft to a previously created version. This action copies the code, dependencies, and secrets from the specified version into the current draft.

        Parameters
        ----------
        id : str
            The unique ID of the module to roll back.

        module_version_id : str
            The unique ID of the module version to roll back to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RollbackActionModuleResponseContent]
            The rollback was successful.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/rollback",
            method="POST",
            json={
                "module_version_id": module_version_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RollbackActionModuleResponseContent,
                    parse_obj_as(
                        type_=RollbackActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawModulesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleListItem, GetActionModulesResponseContent]:
        """
        Retrieve a paginated list of all Actions Modules with optional filtering and totals.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleListItem, GetActionModulesResponseContent]
            The action modules were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "actions/modules",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModulesResponseContent,
                    parse_obj_as(
                        type_=GetActionModulesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.modules
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        per_page=per_page,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        code: str,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        publish: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateActionModuleResponseContent]:
        """
        Create a new Actions Module for reusable code across actions.

        Parameters
        ----------
        name : str
            The name of the action module.

        code : str
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        api_version : typing.Optional[str]
            The API version of the module.

        publish : typing.Optional[bool]
            Whether to publish the module immediately after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateActionModuleResponseContent]
            The action module was created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "actions/modules",
            method="POST",
            json={
                "name": name,
                "code": code,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionModuleSecretRequest], direction="write"
                ),
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionModuleDependencyRequest], direction="write"
                ),
                "api_version": api_version,
                "publish": publish,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionModuleResponseContent,
                    parse_obj_as(
                        type_=CreateActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetActionModuleResponseContent]:
        """
        Retrieve details of a specific Actions Module by its unique identifier.

        Parameters
        ----------
        id : str
            The ID of the action module to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetActionModuleResponseContent]
            The action module was retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionModuleResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Permanently delete an Actions Module. This will fail if the module is still in use by any actions.

        Parameters
        ----------
        id : str
            The ID of the Actions Module to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 412:
                raise PreconditionFailedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        code: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateActionModuleResponseContent]:
        """
        Update properties of an existing Actions Module, such as code, dependencies, or secrets.

        Parameters
        ----------
        id : str
            The ID of the action module to update.

        code : typing.Optional[str]
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateActionModuleResponseContent]
            The action module was updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}",
            method="PATCH",
            json={
                "code": code,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionModuleSecretRequest], direction="write"
                ),
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionModuleDependencyRequest], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionModuleResponseContent,
                    parse_obj_as(
                        type_=UpdateActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_actions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]:
        """
        Lists all actions that are using a specific Actions Module, showing which deployed action versions reference this Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]
            The actions were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/actions",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetActionModuleActionsResponseContent,
                    parse_obj_as(
                        type_=GetActionModuleActionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.actions
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list_actions(
                        id,
                        page=page + 1,
                        per_page=per_page,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def rollback(
        self, id: str, *, module_version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RollbackActionModuleResponseContent]:
        """
        Rolls back an Actions Module's draft to a previously created version. This action copies the code, dependencies, and secrets from the specified version into the current draft.

        Parameters
        ----------
        id : str
            The unique ID of the module to roll back.

        module_version_id : str
            The unique ID of the module version to roll back to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RollbackActionModuleResponseContent]
            The rollback was successful.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/modules/{encode_path_param(id)}/rollback",
            method="POST",
            json={
                "module_version_id": module_version_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RollbackActionModuleResponseContent,
                    parse_obj_as(
                        type_=RollbackActionModuleResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
